import sys

DB_HOST = sys.modules['DB_HOST']

DB_USER = sys.modules['DB_USER']

DB_PASSWORD = sys.modules['DB_PASSWORD']

DB_PORT = sys.modules['DB_PORT']

DB_NAME = sys.modules['DB_NAME']

DB_CHARSET = sys.modules['DB_CHARSET']

TIME_OUT = sys.modules['TIME_OUT']

HEADERS = sys.modules['HEADERS']

PROXIES = sys.modules['PROXIES']

FIREFOX_DRIVER = sys.modules['FIREFOX_DRIVER']

WAIT_TIME = sys.modules['WAIT_TIME']

LOG_DIR = sys.modules['LOG_DIR']



import time
from selenium import webdriver
from selenium.webdriver.common.action_chains import ActionChains
from selenium.webdriver.support.ui import WebDriverWait
from .config import *
from .request import Request

class FireFox():
    """This is a class that simulates the behavior of Firefox browsers."""

    def __init__(self):
        """initialize"""
        options = webdriver.FirefoxOptions()
        options.set_headless()

        req = Request()

        proxy = req.getProxy()
        if len(proxy) > 0:
            proxyStr = list(proxy.values())[0]
            proxyList = proxyStr.split(':')
            options.set_preference('network.proxy.type', 1)
            options.set_preference('network.proxy.http', proxyList[0])
            options.set_preference('network.proxy.http_port', int(proxyList[1]))

        header = req.getHeader()
        if len(header) > 0:
            options.set_preference("general.useragent.override", list(header.values())[0])

        self.browser = webdriver.Firefox(executable_path = FIREFOX_DRIVER, options = options)

    def load(self, url, sleepTime = 0):
        """browser loading page"""
        self.browser.get(url)
        if sleepTime > 0:
            time.sleep(sleepTime)
        else:
            self.browser.implicitly_wait(WAIT_TIME)
        
        self.content = self.browser.page_source

    def refresh(self, sleepTime = 0):
        """browser refresh page"""
        self.browser.refresh()
        if sleepTime > 0:
            time.sleep(sleepTime)
        else:
            self.browser.implicitly_wait(WAIT_TIME)
        
        self.content = self.browser.page_source

    def click(self, elementXpath, waitElementXpath = '', sleepTime = 0):
        """mouse click"""
        element = self.browser.find_element_by_xpath(elementXpath)
        ActionChains(self.browser).click(element).perform()
        if sleepTime > 0:
            time.sleep(sleepTime)
        elif waitElementXpath != '':
            WebDriverWait(self.browser, WAIT_TIME).until(lambda driver : driver.find_element_by_xpath(waitElementXpath))
        self.content = self.browser.page_source

    def moveTo(self, elementXpath, waitElementXpath = '', sleepTime = 0):
        """mouse move to element"""
        element = self.browser.find_element_by_xpath(elementXpath)
        ActionChains(self.browser).move_to_element(element).perform()
        if sleepTime > 0:
            time.sleep(sleepTime)
        elif waitElementXpath != '':
            WebDriverWait(self.browser, WAIT_TIME).until(lambda driver : driver.find_element_by_xpath(waitElementXpath))
        self.content = self.browser.page_source

    def scrollTo(self, xpos, ypos, waitElementXpath = '', sleepTime = 0):
        """window scroll to a location"""
        self.browser.execute_script("window.scrollTo(arguments[0], arguments[1]);", xpos, ypos)
        if sleepTime > 0:
            time.sleep(sleepTime)
        elif waitElementXpath != '':
            WebDriverWait(self.browser, WAIT_TIME).until(lambda driver : driver.find_element_by_xpath(waitElementXpath))
        self.content = self.browser.page_source

    def scrollToBottom(self, waitElementXpath = '', sleepTime = 0):
        """window scroll to the bottom"""
        self.browser.execute_script("window.scrollTo(0, document.body.scrollHeight);")
        if sleepTime > 0:
            time.sleep(sleepTime)
        elif waitElementXpath != '':
            WebDriverWait(self.browser, WAIT_TIME).until(lambda driver : driver.find_element_by_xpath(waitElementXpath))
        self.content = self.browser.page_source

    def setValue(self, elementXpath, value):
        """set the value attribute of the element"""
        element = self.browser.find_element_by_xpath(elementXpath)
        self.browser.execute_script("arguments[0].value = arguments[1];", element, value)
        self.content = self.browser.page_source

    def setText(self, elementXpath, value):
        """set the content of the element"""
        element = self.browser.find_element_by_xpath(elementXpath)
        self.browser.execute_script("arguments[0].innerHTML = arguments[1];", element, value)
        self.content = self.browser.page_source

    def switchToFrame(self, elementXpath, sleepTime = 0):
        """switch to a frame"""
        element = self.browser.find_element_by_xpath(elementXpath)
        self.browser.switch_to.frame(element)
        if sleepTime > 0:
            time.sleep(sleepTime)
        self.content = self.browser.page_source

    def returnToPage(self, sleepTime = 0):
        """return to page"""
        self.browser.switch_to_default_content()
        if sleepTime > 0:
            time.sleep(sleepTime)
        self.content = self.browser.page_source

    def screenShot(self, fileName):
        """screen capture"""
        self.browser.save_screenshot(fileName)

    def quit(self):
        """browser quit"""
        self.browser.quit()



"""Project configuration file."""
import sys


"""Mysql database configuration"""

"""database host name"""
sys.modules['DB_HOST'] = 'localhost'
"""database account"""
sys.modules['DB_USER'] = 'root'
"""database password"""
sys.modules['DB_PASSWORD'] = 'root'
"""database port"""
sys.modules['DB_PORT'] = 3306
"""database name"""
sys.modules['DB_NAME'] = 'spider'
"""database character set"""
sys.modules['DB_CHARSET'] = 'utf8'


"""Http request configuration"""

"""request expiration time, unit: second"""
sys.modules['TIME_OUT'] = 10

"""
proxy User-Agent:
This parameter is a list type, you can configure multiple, randomly select one when accessing.
"""
sys.modules['HEADERS'] = [{'User-Agent':'Mozilla/5.0 (Windows NT 6.1; W) Gecko/20100101 Firefox/59.0'}]

"""
proxy ip:
This parameter is a list type, you can configure multiple, randomly select one when accessing.
"""
#sys.modules['PROXIES'] = [{'http':'61.136.163.245:8103'}]
sys.modules['PROXIES'] = [{}]

"""
absolute path of phantomjs:
If you do not need to get the webpage after javascript is running, you can ignore this.
"""
sys.modules['FIREFOX_DRIVER'] = '/usr/local/bin/geckodriver'

"""webdriver waits for the page to finish loading, unit: second"""
sys.modules['WAIT_TIME'] = 20


"""File generation configuration"""

"""
project execution log directory:
If enabled, to ensure write access to this directory.
"""
sys.modules['LOG_DIR'] = ''



"""
like spider: a crawler framework that allows developers to easily extract web page information and quickly store data.

copyright: (c) 2018 by Yingjie Wu.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.

author: wyjhaha@foxmail.com

github: https://github.com/wuyingjie1002/like_spider

Note: please read the development documentation and sample code in detail during development.
github wiki: https://github.com/wuyingjie1002/like_spider/wiki

If you have any questions or suggestions, please contact me by email.
"""

import time, re, os, pymysql, zipfile
from .config import *
from .request import Request
from .html import Html
from .mysql import Mysql
from .excel import Excel
from .fireFox import FireFox

def filterBlankChar(sourceStr):
    """filter whitespace characters in a string"""
    newStr = re.sub(r'[\f\n\r\t\v]+', '', sourceStr)
    return newStr.strip()

def escapeString(sourceStr):
    """escape special characters"""
    return pymysql.escape_string(sourceStr)

def executeLog(content):
    """project execution log"""
    localTime = time.localtime()
    month = time.strftime('%Y%m', localTime)
    day = time.strftime('%Y%m%d', localTime)
    strTime = time.strftime('%Y-%m-%d %H:%M:%S', localTime)
    if 'LOG_DIR' in globals() and LOG_DIR != "":
        if LOG_DIR[-1] == "/":
            path = LOG_DIR + month
        else:
            path = LOG_DIR + "/" + month
        if os.path.exists(path) == False:
            os.makedirs(path)
        fileName = path + "/" + day + ".log"
        f = open(fileName, 'a+')
        content = strTime + " | " + content + "\n"
        f.write(content)
        f.close()
    else:
        print('file directory error')

def zipCompress(fileList, outputPath):
    """compress the folder into a zip"""
    f = zipfile.ZipFile(outputPath, 'w', zipfile.ZIP_DEFLATED)
    for fl in fileList:
        f.write(fl)
    f.close()

__all__ = ['Request', 'Html', 'Mysql', 'Excel', 'FireFox', 'filterBlankChar', 'escapeString', 'executeLog', 'zipCompress']



import re

class Html():
    """This is a class that extracts the contents of an html tag."""

    def __init__(self):
        self.tagList = []

    def setHtml(self, html = ''):
        """set the html you want to work on now"""
        if html == '':
            print('html param error')
        else:
            self.html = html
            return self

    def setTag(self, tagName = '', setAttr = {}, getAttr = (), haveFoot = True):
        """set the html tag you want to extract"""
        if tagName == '':
            print('tag param error')
        else:
            self.tagName = tagName
            self.setAttr = ''
            self.getAttr = getAttr
            if len(setAttr) > 0:
                setAttrList = []
                for attrIndex in setAttr:
                    setAttrList.append(attrIndex + '\s*=\s*[\'\"]' + setAttr[attrIndex] + '[\'\"]')
                self.setAttr = '[^>]*'.join(setAttrList)
            self.haveFoot = haveFoot
            self.tagHeadPattern = re.compile(r'<\s*'+tagName+'[^>]*>', re.S)
            self.tagFootPattern = re.compile(r'<\s*\/'+tagName+'[^>]*>', re.S)
            lastTagLen = len(self.tagList)
            if lastTagLen > 0:
                lastTagList = self.tagList
                self.tagList = []
                forCount = 0
                for lastTag in lastTagList:
                    if forCount >= lastTagLen:
                        break
                    self.html = lastTag['content']
                    self.handleTag()
                    forCount += 1
            else:
                self.handleTag()
            return self

    def handleTag(self):
        """extracted html tags"""
        if not self.haveFoot:
            pattern = re.compile(r'<\s*'+self.tagName+'[^>]*'+self.setAttr+'[^>]*>', re.S)
            headMatch = pattern.findall(self.html)
            if len(headMatch) > 0:
                for match in headMatch:
                    currTag = {'attr':{}, 'content':''}
                    if len(self.getAttr) > 0:
                        for ga in self.getAttr:
                            attrMatch = re.search(r'<\s*'+self.tagName+'[^>]*'+ga+'\s*=\s*[\'\"](.*?)[\'\"]', match, re.S)
                            if attrMatch != None:
                                currTag['attr'][ga] = attrMatch.group(1)
                    currTag['content'] = match
                    self.tagList.append(currTag)
        else:
            firstHeadMatch = re.search(r'<\s*'+self.tagName+'[^>]*'+self.setAttr+'[^>]*>', self.html, re.S)
            if firstHeadMatch != None:
                firstHead = firstHeadMatch.group()
                firstHeadLen = len(firstHead)
                firstHeadPos = self.html.find(firstHead)
                self.html = self.html[firstHeadPos:]
                footMatch = self.tagFootPattern.findall(self.html)
                if len(footMatch) > 0:
                    start = 0
                    lastStart = 0
                    end = 0
                    currHtml = ''
                    cutNum = 0
                    for index in range(len(footMatch)):
                        if index != 0 and cutNum > 0:
                            cutNum -= 1
                        if index != 0 and cutNum == 0:
                            break
                        start = self.html.find(footMatch[index], end)
                        currHtml += self.html[lastStart:start]
                        cutNum += len(self.tagHeadPattern.findall(self.html[end:start]))
                        footLen = len(footMatch[index])
                        end = start + footLen
                        lastStart = start
                        if cutNum > 1:
                            continue
                        else:
                            currHtml += self.html[start:end]
                            currTag = {'attr':{}, 'content':''}
                            if len(self.getAttr) > 0:
                                for ga in self.getAttr:
                                    attrMatch = re.search(r'<\s*'+self.tagName+'[^>]*'+ga+'\s*=\s*[\'\"](.*?)[\'\"]', currHtml, re.S)
                                    if attrMatch != None:
                                        currTag['attr'][ga] = attrMatch.group(1)
                            content = currHtml[firstHeadLen:(len(currHtml) - footLen)]
                            currTag['content'] = content
                            self.tagList.append(currTag)
                    self.html = self.html[end:]
                    self.handleTag()

    def getData(self):
        """get the extracted content"""
        tagList = self.tagList
        self.tagList = []
        return tagList



import time, os
from openpyxl import load_workbook
from openpyxl import Workbook
from .config import *

class Excel():
    """This is a class that saves data to an excel file."""

    def loadFile(self, fileName):
        """load excel file"""
        self.wb = load_workbook(fileName)
        self.sheets = self.wb.get_sheet_names()

    def loadSheet(self, sheet):
        """load a sheet"""
        self.table = self.wb[sheet]
        self.rows = self.table.max_row
        self.cols = self.table.max_column

    def getValue(self, row, col):
        """get a value"""
        return self.table.cell(row, col).value

    def saveFile(self, data, fileName):
        """save data to an excel file."""
        if fileName == "":
            print('file error')
            return False
        totalRow = len(data)
        if totalRow > 0:
            wb = Workbook()
            ws = wb.active
            for row in range(1, (totalRow + 1)):
                totalCol = len(data[(row - 1)])
                if totalCol > 0:
                    for col in range(1, (totalCol + 1)):
                        cell = ws.cell(row = row, column = col)
                        cell.value = data[(row - 1)][(col - 1)]
                else:
                    print('col data error')
                    break
            if totalCol > 0:
                wb.save(fileName)
        else:
            print('row data error')

    def appendFile(self, data, fileName, sheet = ''):
        """append data to an excel file."""
        if fileName == "":
            print('file error')
            return False
        if os.path.exists(fileName):
            self.loadFile(fileName)
            if sheet == '':
                sheet = self.sheets[0]
            self.loadSheet(sheet)
            if self.rows > 0 and self.cols > 0:
                fileData = []
                for row in range(1, self.rows + 1):
                    rowData = []
                    for col in range(1, self.cols + 1):
                        rowData.append(self.getValue(row, col))
                    fileData.append(rowData)
                fileData.extend(data)
                data = fileData
        self.saveFile(data, fileName)


import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name = 'like_spider',
    version = '1.1.9',
    author = 'Yingjie Wu',
    author_email = 'wyjhaha@foxmail.com',
    description = 'a crawler framework that allows developers to easily extract web page information and quickly store data',
    long_description = long_description,
    long_description_content_type="text/markdown",
    url = 'https://github.com/wuyingjie1002/like_spider',
    packages = setuptools.find_packages(),
    classifiers = (),
    install_requires = [
        'requests',
        'pymysql',
        'openpyxl',
        'selenium'
    ]
)



import requests, random
from .config import *
from selenium import webdriver

class Request():
    """This is a class that handles http requests."""

    def __init__(self):
        """initialize"""
        self.session = requests.Session()

    def getProxy(self):
        """get proxy ip"""
        proxy = {}
        if 'PROXY_API' in globals() and PROXY_API != '':
            req = self.session.get(PROXY_API)
            ipStr = req.text
            if ipStr != '':
                ipList = ipStr.split('--')
                proxy = {ipList[2]: ipList[0] + ':' + ipList[1]}
        else:
            if 'PROXIES' in globals():
                proxyLen = len(PROXIES)
                if proxyLen > 0:
                    proxy = random.sample(PROXIES, 1)[0]
        return proxy

    def getHeader(self):
        """get client information"""
        header = {}
        if 'HEADERS' in globals():
            headerLen = len(HEADERS)
            if headerLen > 0:
                header = random.sample(HEADERS, 1)[0]
        return header

    def get(self, url, data = {}, referer = ''):
        """http get request"""
        header = self.getHeader()
        header['Referer'] = referer
        req = self.session.get(url, data = data, headers = header, proxies = self.getProxy(), timeout = TIME_OUT)

        if req.status_code == 200:
            return req.text
        else:
            print('request status code : ', req.status_code)
            return ''

    def post(self, url, data = {}, referer = ''):
        """http post request"""
        header = self.getHeader()
        header['Referer'] = referer
        req = self.session.post(url, data = data, headers = header, proxies = self.getProxy(), timeout = TIME_OUT)

        if req.status_code == 200:
            return req.text
        else:
            print('request status code : ', req.status_code)
            return ''

    def final(self, url):
        """webdriver loads webpage"""
        options = webdriver.FirefoxOptions()
        options.set_headless()

        proxy = self.getProxy()
        if len(proxy) > 0:
            proxyStr = list(proxy.values())[0]
            proxyList = proxyStr.split(':')
            options.set_preference('network.proxy.type', 1)
            options.set_preference('network.proxy.http', proxyList[0])
            options.set_preference('network.proxy.http_port', int(proxyList[1]))

        header = self.getHeader()
        if len(header) > 0:
            options.set_preference("general.useragent.override", list(header.values())[0])

        browser = webdriver.Firefox(executable_path = FIREFOX_DRIVER, options = options)
        browser.get(url)
        browser.implicitly_wait(WAIT_TIME)
        content = browser.page_source
        browser.quit()

        return content

    def downloadImg(self, url, fileName, referer = ''):
        """download image"""
        header = self.getHeader()
        header['Referer'] = referer
        req = self.session.get(url, headers = header, proxies = self.getProxy(), timeout = TIME_OUT)

        if req.status_code == 200:
            imgFile = open(fileName, 'wb+')
            imgFile.write(req.content)
            imgFile.close()
        else:
            print('request status code : ', req.status_code)



import pymysql, time, os, sys
from .config import *

class Mysql():
    """This is a mysql database class."""

    def __init__(self):
        self.db = pymysql.connect(host = DB_HOST, user = DB_USER, passwd = DB_PASSWORD, db = DB_NAME, port = DB_PORT, charset = DB_CHARSET)
        self.cursor = self.db.cursor()

    def insert(self, sql, data = ()):
        """insert data"""
        return self.execute(1, sql, data)
	
    def update(self, sql, data = ()):
        """update data"""
        return self.execute(2, sql, data)

    def delete(self, sql, data = ()):
        """delete data"""
        return self.execute(3, sql, data)

    def execute(self, operation, sql, data = ()):
        """execute sql"""
        try:
            self.cursor.execute(sql, data)
            if operation == 1:
                insertId = int(self.cursor.lastrowid)
            self.db.commit()
            if operation == 1:
                return insertId
            return True
        except:
            print('ERROR SQL : ', sql, data)
            self.errorLog(sql, data)
            if operation == 1:
                return 0
            return False

    def getRow(self, sql, data = ()):
        """get a row of data"""
        return self.select(1, sql, data)

    def getAll(self, sql, data = ()):
        """get multiple rows of data"""
        return self.select(2, sql, data)

    def getOne(self, sql, data = ()):
        """get a column of data in a row"""
        return self.select(3, sql, data)

    def getCol(self, sql, data = ()):
        """get a column of data"""
        return self.select(4, sql, data)

    def select(self, operation, sql, data):
        """processing query operations"""
        result = ''
        try:
            self.cursor.execute(sql, data)
            if operation == 1:
                data = self.cursor.fetchone()
                if data != None:
                    result = data
            elif operation == 2:
                data = self.cursor.fetchall()
                if len(data) > 0:
                    result = data
            elif operation == 3:
                data = self.cursor.fetchone()
                if data != None:
                    result = data[0]
            else:
                data = self.cursor.fetchall()
                if len(data) > 0:
                    result = []
                    for item in data:
                        result.append(item[0])
        except:
            print('ERROR SQL : ', sql, data)
            self.errorLog(sql, data)
		
        return result

    def errorLog(self, sql, data):
        """sql error log"""
        localTime = time.localtime()
        month = time.strftime('%Y%m', localTime)
        day = time.strftime('%Y%m%d', localTime)
        strTime = time.strftime('%Y-%m-%d %H:%M:%S', localTime)
        if 'LOG_DIR' in globals() and LOG_DIR != "":
            if LOG_DIR[-1] == "/":
                path = LOG_DIR + "sqlErrorLog/" + month
            else:
                path = LOG_DIR + "/sqlErrorLog/" + month
            if os.path.exists(path) == False:
                os.makedirs(path)
            fileName = path + "/" + day + ".log"
            f = open(fileName, 'a+')
            content = strTime + " | " + sql + " | " + ','.join(data) + "\n"
            f.write(content)
            f.close()
        else:
            print('file directory error')
